using System;
using System.Drawing;

using OgreDotNet;

namespace ExtendedCameraSample
{
	/// <summary>
	/// Summary description for ExtendedCamera.
	/// </summary>
	public class ExtendedCamera
	{
		public enum Mode : uint
		{ 
			Chasing = 0, Fixed = 1, FirstPerson = 2
		}

		protected SceneNode mTargetNode;
		protected SceneNode mCameraNode;
		protected Camera mCamera;

		protected SceneManager mSceneMgr;
		protected string mName;

		protected bool mOwnCamera;
		protected float mTightness;

		protected RenderWindow mRenderWindow = null;
		protected Viewport mViewport = null;

		public ExtendedCamera(String name, SceneManager sceneMgr, Camera camera)
		{
			// Basic member references setup
			mName = name;
			mSceneMgr = sceneMgr;

			// Create the camera's node structure
			mCameraNode = mSceneMgr.GetRootSceneNode().CreateChildSceneNode(mName);
			mTargetNode = mSceneMgr.GetRootSceneNode().CreateChildSceneNode(mName + "_target");
			mCameraNode.SetAutoTracking(true, mTargetNode); // The camera will always look at the camera target
			mCameraNode.SetFixedYawAxis(true); // Needed because of auto tracking

			// Create our camera if it wasn't passed as a parameter 
			if (camera == null) 
			{
				mCamera = mSceneMgr.CreateCamera(mName);
				mOwnCamera = true;
			}
			else 
			{
				mCamera = camera;
				mOwnCamera = false;
			}
			// ... and attach the Ogre camera to the camera node
			mCameraNode.AttachObject(mCamera);

			// Default tightness
			mTightness = 0.01f;

		}

		public void setTightness (float tightness) 
		{
			mTightness = tightness;
		}

		public float getTightness () 
		{
			return mTightness;
		}

		public Math3D.Vector3 getCameraPosition () 
		{
			return mCameraNode.GetPosition();
		}

		public void instantUpdate (Math3D.Vector3 cameraPosition, Math3D.Vector3 targetPosition) 
		{
			mCameraNode.SetPosition (cameraPosition);
			mTargetNode.SetPosition (targetPosition);
		}

		public void update (float elapsedTime, Math3D.Vector3 cameraPosition, Math3D.Vector3 targetPosition) 
		{
			// Handle movement
			Math3D.Vector3 displacement;

			displacement = (cameraPosition - mCameraNode.GetPosition ()) * mTightness;
			mCameraNode.Translate (displacement);

			displacement = (targetPosition - mTargetNode.GetPosition ()) * mTightness;
			mTargetNode.Translate (displacement);
		}

		public Camera getCamera()
		{
			return mCamera;
		}


	}
}
