using System;
using System.Drawing;

using OgreDotNet;

using Math3D;

namespace ExtendedCameraSample
{
	/// <summary>
	/// Summary description for Class1.
	/// </summary>
	class ExtendedCameraSampleMain : ExampleApplication
	{
		protected Ogre mOgre = null;
		protected ExtendedCamera exCamera = null;
		protected OgreCharacter mOgreCharacter = null;
		protected ExtendedCamera mExCamera = null;

		protected bool mForward = false;
		protected bool mBackward = false;
		protected bool mLeft = false;
		protected bool mRight = false;

		protected override void CreateScene()
		{
			mOgreCharacter = new OgreCharacter("Ogre 1", mSceneManager,new Math3D.Vector3(20,20,400));
			
			mSceneManager.RemoveAllCameras();
			mRenderWindow.RemoveAllViewports();
			mExCamera = new ExtendedCamera("MainCamera", mSceneManager,null);

			// only necessary if the ExtendedCamera constructor has created a new basic OgreDotNet.Camera (you have passed a null value)
			mExCamera.getCamera().NearClipDistance = 5;
			mViewport = mRenderWindow.AddViewport( mExCamera.getCamera() );
			mViewport.BackgroundColor = Color.Blue;
			mExCamera.getCamera().AspectRatio = (float)mViewport.ActualWidth/(float)mViewport.ActualHeight;

			mOgre = new Ogre();
			mOgre.setCharacter(mOgreCharacter);
			mOgre.setExtendedCamera(mExCamera);

			Plane plane = new Plane(Vector3.UnitY,0);
			MeshManager.GetSingleton().CreatePlane("ground",ResourceGroupManager.DEFAULT_RESOURCE_GROUP_NAME.ToString(),plane,1500,1500,20,20,true,1,5,5,Vector3.UnitZ);

			Entity planeEntity = mSceneManager.CreateEntity("GroundEntity","ground");
			mSceneManager.GetRootSceneNode().CreateChildSceneNode().AttachObject(planeEntity);
			planeEntity.SetMaterialName("Examples/Rockwall");
		}

		protected override bool FrameStarted(FrameEvent e)
		{
			bool result = base.FrameStarted (e);

			mOgreCharacter.Forward(mForward);
			mOgreCharacter.Backward(mBackward);
			mOgreCharacter.Left(mLeft);
			mOgreCharacter.Right(mRight);

			bool result2 = mOgre.Update(mDeltaTime);

			return result && result2;
		}

		protected override void MouseMotion(MouseMotionEvent e)
		{
			//base.MouseMotion (e);
		}

		protected override void KeyClicked(KeyEvent e)
		{
			base.KeyClicked (e);

			switch (e.KeyCode)
			{
				case KeyCode.F1:
					mOgre.setCameraMode(ExtendedCamera.Mode.FirstPerson);
					break;
				case KeyCode.F2:
					mOgre.setCameraMode(ExtendedCamera.Mode.Fixed);
					break;
				case KeyCode.F3:
					mOgre.setCameraMode(ExtendedCamera.Mode.Chasing);
					break;
			}
		}

		protected override void KeyPressed(KeyEvent e)
		{
			base.KeyPressed (e);

			switch (e.KeyCode)
			{
				case KeyCode.W:
					mForward = true;
					break;
				case KeyCode.S:
					mBackward = true;
					break;
				case KeyCode.A:
					mLeft = true;
					break;
				case KeyCode.D:
					mRight = true;
					break;
			}
		}

		protected override void KeyReleased(KeyEvent e)
		{
			base.KeyPressed (e);

			switch (e.KeyCode)
			{
				case KeyCode.W:
					mForward = false;
					break;
				case KeyCode.S:
					mBackward = false;
					break;
				case KeyCode.A:
					mLeft = false;
					break;
				case KeyCode.D:
					mRight = false;
					break;
			}
		}



		/// <summary>
		/// The main entry point for the application.
		/// </summary>
		[STAThread]
		static void Main(string[] args)
		{
			using(ExtendedCameraSampleMain  app = new ExtendedCameraSampleMain() )
			{
				app.Start();
			}
		}
	}
}
