using System;
using OgreDotNet;

namespace ExtendedCameraSample
{
	/// <summary>
	/// Summary description for OgreCharacter.
	/// </summary>
	public class OgreCharacter : Character
	{
		protected string mName;

		protected bool mForward = false;
		protected bool mBackward = false;
		protected bool mLeft = false;
		protected bool mRight = false;

		public OgreCharacter(string name,SceneManager sceneMgr, Math3D.Vector3 initialPosition)
		{
			mName = name;
			mSceneMgr = sceneMgr;

			mMainNode = mSceneMgr.GetRootSceneNode().CreateChildSceneNode(mName,initialPosition);
			mSightNode = mMainNode.CreateChildSceneNode(mName+"_sight",new Math3D.Vector3(0,0,100));
			mCameraNode = mMainNode.CreateChildSceneNode(mName+"_camera",new Math3D.Vector3(0,50,-100));

			mEntity = mSceneMgr.CreateEntity (mName, "OgreHead.mesh");
			mMainNode.AttachObject(mEntity);

		}

		//public override void update (float elapsedTime, InputReader input) 
		public override void update (float elapsedTime) 
		{
			// Handle movement
			if (mForward) 
			{
				mMainNode.Translate (mMainNode.GetOrientation () * (new Math3D.Vector3 (0, 0, 100 * elapsedTime)));
			}
			if (mBackward) 
			{
				mMainNode.Translate (mMainNode.GetOrientation () * (new Math3D.Vector3 (0, 0, -50 * elapsedTime)));
			}
			if (mLeft) 
			{
				mMainNode.Yaw (new Radian (2 * elapsedTime));
			}
			if (mRight) 
			{
				mMainNode.Yaw (new Radian (-2 * elapsedTime));
			}

			mForward = false;
			mBackward = false;
			mLeft = false;
			mRight = false;
		}

		public void setVisible (bool visible) 
		{
			mMainNode.SetVisible (visible);
		}

		public void Forward(bool forward)
		{
			mForward = forward;
		}

		public void Backward(bool backward)
		{
			mBackward = backward;
		}

		public void Left(bool left)
		{
			mLeft = left;
		}

		public void Right(bool right)
		{
			mRight = right;
		}

	}
}
